# -*- coding: utf-8 -*-

from conrad.utils import get_address


NYC_ADDRESS = {
    "amenity": "New York City Hall",
    "house_number": "260",
    "road": "Broadway",
    "neighbourhood": "Civic Center",
    "suburb": "Manhattan",
    "city": "Manhattan Community Board 1",
    "county": "New York County",
    "state": "New York",
    "postcode": "10000",
    "country": "United States of America",
    "country_code": "us",
    "latitude": 40.7127281,
    "longitude": -74.0060152,
}


def test_bad_place():
    place = "doesnotexist"
    address = get_address(place)
    assert address is None


def test_good_place():
    place = "New York"
    address = get_address(place)
    assert address == NYC_ADDRESS



# -*- coding: utf-8 -*-

import os

from click import get_app_dir

from .__version__ import __version__


CONRAD_HOME = get_app_dir("conrad")
SQL_ALCHEMY_CONN = f"sqlite:///{CONRAD_HOME}/conrad.db"

if not os.path.exists(CONRAD_HOME):
    os.makedirs(CONRAD_HOME)



# -*- coding: utf-8 -*-

import os
import datetime as dt

import pandas

from ..base import BaseCrawler


class PyConCrawler(BaseCrawler):
    def get_events(self):
        df = pandas.read_csv(
            "https://raw.githubusercontent.com/python-organizers/conferences/refs/heads/main/2025.csv",
            quoting=1,
            encoding="utf-8",
            dtype=str,
        )
        df = df.fillna("")

        for event in df.to_dict(orient="records"):
            location = event["Location"].split(",")
            city = state = country = None
            if len(location) == 2:
                city = location[0].strip()
                country = location[1].strip()
            elif len(location) == 3:
                city = location[0].strip()
                state = location[1].strip()
                country = event["Country"]

            cfp_end_date = (
                event["Talk Deadline"] if event["Talk Deadline"] else "1970-01-01"
            )
            cfp_open = (
                True
                if dt.datetime.now() <= dt.datetime.strptime(cfp_end_date, "%Y-%m-%d").replace(hour=23, minute=59, second=59, microsecond=999999)
                else False
            )
            e = {
                "name": event["Subject"],
                "url": event["Website URL"],
                "city": city,
                "state": state,
                "country": country,
                "location": ", ".join(
                    filter(lambda x: x is not None, [city, state, country])
                ),
                "cfp_open": cfp_open,
                "cfp_end_date": cfp_end_date,
                "start_date": event["Start Date"],
                "end_date": event["End Date"],
                "source": "https://github.com/python-organizers/conferences",
                "tags": ["python"],
                "kind": "conference",
                "by": "bot",
            }
            self.events.append(e)



# -*- coding: utf-8 -*-

import datetime as dt

import json
import requests
from bs4 import BeautifulSoup

from ..base import BaseCrawler


class PyDataEvent:
    def __init__(self, name, city, country, location, start_date, end_date, url):
        self.name = name
        self.url = url
        self.city = city
        self.state = None
        self.country = country
        self.location = location
        self.cfp_open = False
        self.cfp_end_date = "1970-01-01"
        self.start_date = start_date
        self.end_date = end_date
        self.source = PyDataCrawler.URL
        self.tags = ["python", "pydata"]
        self.kind = "conference"
        self.by = "bot"

    def to_json(self):
        return {
            "name": self.name,
            "url": self.url,
            "city": self.city,
            "state": self.state,
            "country": self.country,
            "location": self.location,
            "cfp_open": self.cfp_open,
            "cfp_end_date": self.cfp_end_date,
            "start_date": self.start_date.strftime(PyDataCrawler.DATE_FORMAT),
            "end_date": self.end_date.strftime(PyDataCrawler.DATE_FORMAT),
            "source": self.source,
            "tags": self.tags,
            "kind": self.kind,
            "by": self.by,
        }


class PyDataCrawler(BaseCrawler):
    URL = "https://pydata.org/event-schedule/"
    HEADERS = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/50.0.2661.102 Safari/537.36"
    }
    SELECTOR = 'div.mec-event-list-classic > script[type="application/ld+json"]'
    DATE_FORMAT = "%Y-%m-%d"

    def _format_date(self, date_str):
        return dt.datetime.strptime(date_str, self.DATE_FORMAT).date()

    def _parse_pydata_event(self, event_article):
        content = json.loads(event_article.decode_contents())

        try:
            location = content.get("location", {}).get("name")
            city, country = location.split(", ")
        except (KeyError, ValueError, AttributeError):
            city = country = None

        return PyDataEvent(
            name=content["name"],
            url=content["url"],
            city=city,
            country=country,
            location=location,
            start_date=self._format_date(content["startDate"]),
            end_date=self._format_date(content["endDate"]),
        )

    def _parse_pydata_events(self, event_articles):
        return list(map(self._parse_pydata_event, event_articles))

    def get_events(self):
        resp = requests.get(self.URL, headers=self.HEADERS)
        page = BeautifulSoup(resp.content, features="html.parser")
        pydata_events = self._parse_pydata_events(page.select(self.SELECTOR))
        self.events = [event.to_json() for event in pydata_events]



# -*- coding: utf-8 -*-

from .pycon.pycon_crawler import PyConCrawler
from .python.python_crawler import PythonCrawler
from .pydata.pydata_crawler import PyDataCrawler
from .confs_tech.confs_tech_crawler import ConfsTechCrawler
from .italy.italy_crawler import ItalyCrawler
from .papercall.papercall_crawler import PapercallCrawler



# -*- coding: utf-8 -*-

import datetime as dt
import json
import logging
import os
import sys
from collections import Counter

import geopy.exc as geopyexceptions
import requests
from geopy.extra.rate_limiter import RateLimiter
from geopy.geocoders import Nominatim
from packaging import version

from . import CONRAD_HOME, __version__
from .db import engine
from .schema import *

SELFCHECK_DATE_FMT = "%Y-%m-%dT%H:%M:%SZ"


logger = logging.getLogger(__name__)


# https://github.com/pypa/pip/blob/master/src/pip/_internal/self_outdated_check.py
class SelfCheckState(object):
    def __init__(self, cache_dir):
        self.state = {}
        self.statefile_path = os.path.join(cache_dir, "selfcheck.json")

        # Try to load the existing state
        try:
            with open(self.statefile_path, "r") as f:
                self.state = json.load(f)
        except (IOError, ValueError, KeyError, FileNotFoundError):
            # Explicitly suppressing exceptions, since we don't want to
            # error out if the cache file is invalid.
            pass

    def save(self, pypi_version, current_time):
        # If we do not have a path to cache in, don't bother saving.
        if not self.statefile_path:
            return

        state = {
            "last_check": current_time.strftime(SELFCHECK_DATE_FMT),
            "pypi_version": pypi_version,
        }

        text = json.dumps(state, sort_keys=True, separators=(",", ":"))

        with open(self.statefile_path, "w") as f:
            f.write(text)


def get_pypi_version():
    url = "https://pypi.org/pypi/conference-radar/json"
    response = requests.get(url)
    if response:
        data = response.json()
        pypi_version = data["info"]["version"]
        return pypi_version


def conrad_self_version_check():
    pypi_version = None

    try:
        state = SelfCheckState(cache_dir=CONRAD_HOME)

        current_time = dt.datetime.utcnow()
        # Determine if we need to refresh the state
        if "last_check" in state.state and "pypi_version" in state.state:
            last_check = dt.datetime.strptime(
                state.state["last_check"], SELFCHECK_DATE_FMT
            )
            if (current_time - last_check).total_seconds() < 7 * 24 * 60 * 60:
                pypi_version = state.state["pypi_version"]

        # Refresh the version if we need to or just see if we need to warn
        if pypi_version is None:
            pypi_version = get_pypi_version()

            # Save that we've performed a check
            state.save(pypi_version, current_time)

        conrad_version = version.parse(__version__)
        remote_version = version.parse(pypi_version)

        if conrad_version < remote_version:
            pip_cmd = "{} -m pip".format(sys.executable)
            logger.warning(
                f"You are using conrad version {__version__}; however,"
                f" version {pypi_version} is available.\n"
                "You should consider upgrading with"
                f" '{pip_cmd} install --upgrade conference-radar'."
            )
    except Exception:
        logger.debug(
            "There was an error checking the latest version of conrad",
            exc_info=True,
        )


def initialize_database():
    from .models import Base

    Base.metadata.create_all(engine)


def reset_database():
    from .models import Base

    Base.metadata.drop_all(engine)
    initialize_database()


def get_address(place):
    geolocator = Nominatim(user_agent="conrad")
    geocode = RateLimiter(geolocator.geocode, min_delay_seconds=5)

    address = None
    try:
        location = geolocator.geocode(place)
        if location is not None:
            address = geolocator.reverse(
                "{lat}, {lon}".format(lat=location.latitude, lon=location.longitude)
            ).raw["address"]
            address["latitude"] = location.latitude
            address["longitude"] = location.longitude
    except geopyexceptions.GeocoderTimedOut:
        # TODO: add 2 retries using tenacity
        print(f"Geocoder timed out for {place}!")

    return address


def apply_schema(events, version=LATEST):
    schema = eval(f"v{version}")
    _events = []

    for event in events:
        _event = dict({k: v for k, v in event.items() if k in schema.keys()})
        _events.append(_event)

    return _events


def validate_events(input_events, version=LATEST):
    schema = eval(f"v{version}")
    failures = []

    # check for duplicates
    ie_names = [ie["name"].replace(" ", "").lower() for ie in input_events]
    duplicate_events = [
        event for event, count in Counter(ie_names).items() if count > 1
    ]
    if duplicate_events:
        failures.append(f"Duplicate events found: {duplicate_events}")

    # check if keys exist
    for ie in input_events:
        if set(schema.keys()).difference(set(ie.keys())):
            failures.append("Required fields not found")
            break

    return failures


def mkdir(directory):
    if not os.path.exists(directory):
        os.makedirs(directory)



# -*- coding: utf-8 -*-

from sqlalchemy import Boolean, Column, DateTime, String, Text, ForeignKey
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
ID_LEN = 100
STR_LEN = 1024


class Event(Base):
    __tablename__ = "event"

    id = Column(String(ID_LEN), primary_key=True)
    name = Column(String(STR_LEN))
    url = Column(String(STR_LEN))
    city = Column(String(STR_LEN))
    state = Column(String(STR_LEN))
    country = Column(String(STR_LEN))
    location = Column(String(STR_LEN))
    cfp_open = Column(Boolean, default=False)
    cfp_end_date = Column(DateTime)
    start_date = Column(DateTime)
    end_date = Column(DateTime)
    source = Column(String(STR_LEN))
    tags = Column(Text)
    kind = Column(String(STR_LEN))
    by = Column(String(STR_LEN))


class Reminder(Base):
    __tablename__ = "reminder"

    id = Column(String(ID_LEN), ForeignKey("event.id"), primary_key=True)



# -*- coding: utf-8 -*-

import os
import json
import datetime as dt

from bs4 import BeautifulSoup
from googleapiclient.discovery import build
from google.oauth2.service_account import Credentials

from ..base import BaseCrawler


class PythonCrawler(BaseCrawler):
    def get_events(self):
        credentials = Credentials.from_service_account_file(
            "google_service_account_credentials.json",
            scopes=[
                "https://www.googleapis.com/auth/calendar",
                "https://www.googleapis.com/auth/calendar.readonly",
            ],
        )
        service = build("calendar", "v3", credentials=credentials)

        start_time = dt.datetime.now().strftime("%Y-%m-%dT00:00:00.000Z")
        end_time = None
        events = []

        try:
            page_token = None
            while True:
                event_page = (
                    service.events()
                    .list(
                        singleEvents="False",
                        orderBy="startTime",
                        calendarId="j7gov1cmnqr9tvg14k621j7t5c@group.calendar.google.com",
                        pageToken=page_token,
                        timeMin=start_time,
                        timeMax=end_time,
                    )
                    .execute()
                )
                events.extend([event for event in event_page["items"]])

                page_token = event_page.get("nextPageToken")
                if not page_token:
                    break
        except AccessTokenRefreshError:
            print(
                "The credentials have been revoked or expired, please re-run"
                " the application to re-authorize."
            )

        for event in events:
            if any(
                [word in event["summary"].lower() for word in ["cancel", "postpone"]]
            ):
                continue

            try:
                soup = BeautifulSoup(event["description"], "html.parser")
                event_url = soup.find_all("a", href=True)[0]["href"]
            except (KeyError, IndexError):
                event_url = None

            if "date" in event["start"]:
                start_date = event["start"]["date"]
                end_date = event["end"]["date"]
            elif "dateTime" in event["start"]:
                start_date = event["start"]["dateTime"].split("T")[0]
                end_date = event["end"]["dateTime"].split("T")[0]
            else:
                raise ValueError("Event date not found!")

            e = {
                "name": event["summary"],
                "url": event_url,
                "city": None,
                "state": None,
                "country": None,
                "location": event.get("location"),
                "cfp_open": False,
                "cfp_end_date": "1970-01-01",
                "start_date": start_date,
                "end_date": end_date,
                "source": "https://wiki.python.org/moin/PythonEventsCalendar",
                "tags": ["python"],
                "kind": "conference",
                "by": "bot",
            }
            self.events.append(e)



# -*- coding: utf-8 -*-

import os

from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(here, "conrad", "__version__.py"), "r") as f:
    exec(f.read(), about)

with open("README.md", "r") as f:
    readme = f.read()


requires = [
    "Click>=7.0",
    "cli-helpers>=1.2.1",
    "colorama>=0.4.1",
    "geopy>=2.0.0",
    "requests>=2.22.0",
    "rich>=10.6.0",
    "SQLAlchemy>=1.3.10",
    "textdistance>=4.1.5",
]
dev_requires = ["Sphinx>=2.2.1", "pytest>=3.8.0"]
crawl_requires = [
    "beautifulsoup4>=4.8.1",
    "Cerberus>=1.3.2",
    "dateparser>=0.7.2",
    "GitPython>=3.1.18",
    "google-api-python-client>=1.10.0",
    "pandas>=0.25.2",
    "pyOpenSSL>=19.0.0",
    "requests>=2.22.0",
    "textdistance>=4.1.5",
]
dev_requires = dev_requires + requires
all_requires = crawl_requires + dev_requires


def setup_package():
    metadata = dict(
        name=about["__title__"],
        version=about["__version__"],
        description=about["__description__"],
        long_description=readme,
        long_description_content_type="text/markdown",
        url=about["__url__"],
        author=about["__author__"],
        author_email=about["__author_email__"],
        license=about["__license__"],
        packages=find_packages(exclude=("tests",)),
        install_requires=requires,
        extras_require={"all": all_requires, "dev": dev_requires},
        entry_points={"console_scripts": ["conrad = conrad.__main__:main"]},
        classifiers=[
            # Trove classifiers
            # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
            "License :: OSI Approved :: Apache Software License",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
    )

    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup

    setup(**metadata)


if __name__ == "__main__":
    setup_package()
